#!/usr/bin/python

# Lefromoma v 0.3 - 16/10/2006

###################################################################
# Customize the frontend
###################################################################

# directories and mame binary
xmamebin = "/usr/bin/xmame"
romsdir = "/opt/roms"
snapsdir = "/home/mame/pics"

# text strings
roms_list_text = "All games"
fav_list_text = "My Favorites"

# xmame command line options
# the -s option (1 or 2) is added in the program
# the snapshot directory is also add automatically
options = '-nolcf --fullscreen -ka'

# 800,600 is the ideal size because of the background image used.
# Note: if xmame gives you a blank screen, uncomment 'load extmod' in your
# xorg.conf file
fullscreen = 0
size = width, height =  800 , 600 

# 1 = romslist; 2 favorites list
default_list = 1 

# key repeat; when you hold up or down, you can
# scroll faster through the list. Watch out holding other
# buttons (delay in miliseconds)
keyrepeat = 100

# mouse visible or not in pygame window
# 1 = visible, 0 = no mouse
mouse_visible = 0

# sound ? 1= yes, 0=no
play_sound = 1

# shutdown option ?
shutdown = 1 

###################################################################
# Don't change anything below, unless you know what you're doing
###################################################################

import os
import sys, pygame
from pygame.locals import * 

###################################################################
# Check if dir/file exists. Die if it doesn't
# also check if roms dir is writable
###################################################################

def check_dir_file(tocheck,string,what,writable):
  if what == 1:
    check = os.path.isfile(tocheck)
  else:
    check = os.path.exists(tocheck)
  if check == False:
    print string +' doesn\'t seem to exists. Check settings.'
    os._exit(1)
  # check for writable if var == 1
  if writable == 1:
    write_ok = os.access(tocheck, os.W_OK)
    if write_ok == False:
      print string +' isn\' writable. Check permissions.'
      os._exit(1)

###################################################################
# function to check for complete/favorites/ignore dir
# create them if needed and create symlinks in complete dir
###################################################################

def check_indir(dir,sym):
  check = os.path.exists(dir)
  if check == False:
    print dir +' doesn\'t seem to exists. creating.'
    # check for sym first, read in dir, after that create dir
    if sym == 1:
      print 'Creating symlinks and scaling files in complete dir.'
      romslist_temp = os.popen("ls -1 "+ romsdir)
      os.mkdir(dir)
      scaling = romsdir +"/scaling"
      os.mkdir(scaling)
      for l in romslist_temp.xreadlines():
        cmd = "ln -s "+romsdir+"/"+l.strip()+" "+romsdir+"/complete/"+l
        cmd2 = "ln -s "+romsdir+"/"+l.strip()+" "+romsdir+"/scaling/"+l.strip()+"_2"
        os.popen(cmd)
        os.popen(cmd2)
    # no symlink, so go
    else:
      os.mkdir(dir)

###################################################################
# load background image
###################################################################

def load_background(screen,png):
  background = pygame.image.load(png).convert()
  screen.blit(background, (0,0))

###################################################################
# load snapshot image
###################################################################

def load_snapshot(screen,png):
  
  snapshot = pygame.image.load(png).convert()
  snapposx = snapshot.get_rect(centerx=screen.get_width()/2)
  snapposy = snapshot.get_rect(centery=screen.get_height()/2)
  snappos = []
  snappos.append(snapposx[0])
  snappos.append(snapposy[1])
  screen.blit(snapshot,snappos)

###################################################################
# position text
###################################################################

def position_text(screen,text,pos):
  font_file = snapsdir+'/head0865.ttf'
  font = pygame.font.Font(font_file,16)
  text = font.render(text, 1, (124,195,255))
  textpos = text.get_rect(centerx=screen.get_width()/2)
  textpos[1] = screen.get_height() - pos
  screen.blit(text,textpos)

###################################################################
# show game
###################################################################

def show_screen(screen,list,count,offset,listtext,focus,focustext):

  # load background
  backpng = snapsdir+'/lefromoma.png'
  load_background(screen,backpng)

  # load snapshot image
  png = snapsdir+'/'+list[offset]+'.png'
  check_file = os.path.isfile(png)
  if check_file == False:
    png = snapsdir+'/no_snap_image.png'
  load_snapshot(screen,png)

  # which list and game
  if pygame.font:
    #list_text = listtext + ' - Total : '+ str(len(list) / 2)
    #list_text += ' - Game number ' + str((offset / 2) + 1)
    # which list
    position_text(screen,listtext,590)     
    # game name
    plus = offset + 1
    game = list[plus]
    if listtext == fav_list_text:
      game = game.split('(')
      game = game[0]
    position_text(screen,game,560)
    # game number
    total = str(len(list) / 2)
    gamenr = str((offset / 2) + 1)
    number = gamenr+'/'+total
    position_text(screen,number,530)
    # focustext ?
    if focustext != 0: position_text(screen,focustext,56)

  pygame.display.update()

###################################################################
# start game alright!
###################################################################

def start_game(screen,offset,list,list_len,text,focus):
  # get scaling
  tocheck = romsdir+"/scaling/"+list[offset]+".zip_2"
  check = os.path.isfile(tocheck)
  if check == False:
    scaling = 1
  else:
    scaling = 2

  zip = romsdir+'/'+list[offset]+'.zip'
  cmd = xmamebin+' '+options+' -snapshot_directory '+snapsdir+'   -s '+str(scaling)+' '+zip
  # untoggle lefromoma fullscreen so mame can get focus
  # afterwards go full again (if lefromoma runs fullscreen of course)
  if fullscreen == 1:
    screen = pygame.display.set_mode((size))
    pygame.display.update()
  os.system(cmd)
  if fullscreen == 1:
    screen = pygame.display.set_mode((size),FULLSCREEN)
    show_screen(screen,list,list_len,offset,text,focus,0)

###################################################################
# create roms-, favorites-list list
###################################################################

def create_list(dir):
  list = []

  # Create temporary list of all games from xmame
  gameslist_temp = os.popen(xmamebin + " -lf -nolcf")
  # Create temporary list of roms
  romslist_temp = os.popen("ls -1 "+ dir)

  # Run through romslist_temp, strip .zip populate new romslist
  romslist = []
  for l in romslist_temp.xreadlines():
    new = l.split('.zip')
    romslist.append(new[0])

  # Run through gameslist, split on tab
  # and create two new lists, one with rom, one with full name
  gameslist = []
  nameslist = []
  for l in gameslist_temp.xreadlines():
    new = l.split(' ',1)
    if (len(new) == 2):
      gameslist.append(new[0])
      name = new[1].strip()
      name = name.strip('"')
      nameslist.append(name)

  # Compare romslist with gameslist;
  # when match, add to list
  for i in range(len(romslist)):
    game = romslist[i]
    try: 
      match = gameslist.index(game)
      list.append(game)
      name = nameslist[match]
      list.append(name)
    except:
      print game +" not found (check roms name with official list, rename it if needed)"

  # return it
  return list

###################################################################
# sound (taken from the chimp tutorial)
###################################################################

def load_sound(name,dir):
  class NoneSound:
    def play(self): pass
  if not pygame.mixer or not pygame.mixer.get_init():
    return NoneSound()
  fullname = dir+'/'+name
  try:
    sound = pygame.mixer.Sound(fullname)
  except pygame.error, message:
    print 'Cannot load sound:', fullname
    raise SystemExit, message
  return sound

###################################################################
# before really starting, let's check if the variables are ok
###################################################################

# check for xmame binary
check_dir_file(xmamebin,'Xmame binary',1,0)
# check for no_snap_image
check_dir_file(snapsdir+'/no_snap_image.png','no_snap_image.png',1,0)
# check for roms dir and if it's writable
check_dir_file(romsdir,'Roms directory',0,1)
# check for snaps dir
check_dir_file(snapsdir,'Snapshot directory',0,1)

###################################################################
# Check for ignore/complete/favorites dir, create them if needed
# and populate complete dir with symlinks
###################################################################

complete = romsdir+'/complete'
favorites = romsdir+'/favorites'
ignore = romsdir+'/ignore'

check_indir(complete,1)
check_indir(favorites,0)
check_indir(ignore,0)

###################################################################
# All ok? Start frontend
###################################################################

def main():

  pygame.init()

  if fullscreen == 1:
    screen = pygame.display.set_mode((size),FULLSCREEN)
  else:
    screen = pygame.display.set_mode((size))
  pygame.display.set_caption('Lefromoma')
  pygame.mouse.set_visible(mouse_visible)
  pygame.key.set_repeat(keyrepeat)

  # temp fill with 'loading' on it
  png = snapsdir+'/lefromoma.png'
  load_background(screen,png)
  if pygame.font:
    position_text(screen,"Loading lefromoma",580)
  pygame.display.update()

  # prepare sound (if enabled)
  if play_sound == 1:
    button_sound = load_sound('beep.wav',snapsdir)
    startup_sound = load_sound('startup.wav',snapsdir)
    startgame_sound = load_sound('startgame.wav',snapsdir)
  
  # startup sound
  if play_sound == 1: startup_sound.play()

  # create lists from roms, favorites directory
  roms_list = create_list(complete)
  fav_list = create_list(favorites) 

  # global vars for running (also first run)
  romslist_len = len(roms_list)
  favlist_len = len(fav_list)
  if default_list == 1:
    list_to_read = roms_list
    list_len = romslist_len
    text = roms_list_text
    listr = 'complete'
  if default_list == 2:
    list_to_read = fav_list
    list_len = favlist_len
    text = fav_list_text
    listr = 'favorites'

  first_run = 1
  offset = 0
  ok = 0

  # where are we: array with list places where the focus is
  # scaling factor, default on 2
  # so lefromoma knows what to do

  waw = ['list','changelist','scaling','moverom','ignorerom']
  waw_text = ['','Change list','Scaling','Move game','Ignore game']
  if shutdown == 1: waw.append('shutdown')
  if shutdown == 1: waw_text.append('Shutdown machine')
  wawfocus = 0
  waw_len = len(waw)

  while 1:
    event = pygame.event.wait()
    if event.type == KEYDOWN:

      # --------------------------------------------------
      # enter key ? 
      if event.key == K_RETURN:

        # --------------------------------------------------
        # start game
        if wawfocus == 0:
          if play_sound == 1: startgame_sound.play()
          start_game(screen,offset,list_to_read,list_len,text,wawfocus)
 
        # --------------------------------------------------
        # changing list
        if wawfocus == 1:
          if listr == 'complete' and favlist_len != 0:
            listr = 'favorites'
            list_to_read = fav_list
            list_len = favlist_len
            text = fav_list_text
            offset = 0
          else:
            listr = 'complete'
            list_to_read = roms_list
            list_len = romslist_len
            text = roms_list_text
            offset = 0
          if play_sound == 1: button_sound.play()
          wawfocus = 0
          show_screen(screen,list_to_read,list_len,offset,text,wawfocus,0)

        # --------------------------------------------------
        # Scaling 
        if wawfocus == 2:
          old = scaling
          scaling = scaling + 1
          if scaling == 3: scaling = 1
	  cmd = "mv "+romsdir+"/scaling/"+list_to_read[offset]+".zip_"+str(old)+" "+romsdir+"/scaling/"+list_to_read[offset]+".zip_"+str(scaling)
          os.popen(cmd)
          focustext = waw_text[wawfocus]
	  focustext += ': '+ str(scaling) 
          show_screen(screen,list_to_read,list_len,offset,text,wawfocus,focustext)

        # --------------------------------------------------
        # move rom
        if wawfocus == 3:
          # this could be shorter but I'm ok with it now
          # move rom to favorites directory
          if listr == 'complete':
            temp_int = offset
            game = list_to_read[temp_int]
            roms_list.pop(temp_int) 
            title = list_to_read[temp_int]
            roms_list.pop(temp_int) 
            cmd = 'mv '+complete+'/'+game+'.zip '+favorites
            os.popen(cmd)
            fav_list.append(game)
            fav_list.append(title)
            favlist_len = favlist_len + 2
            romslist_len = romslist_len - 2
            list_len = romslist_len
            offset = offset - 2
          # move rom to roms directory
          if listr == 'favorites':
            temp_int = offset
            game = list_to_read[temp_int]
            fav_list.pop(temp_int)
            title = list_to_read[temp_int]
            fav_list.pop(temp_int)
            cmd = 'mv '+favorites+'/'+game+'.zip '+complete
            os.popen(cmd)
            roms_list.append(game)
            roms_list.append(title)
            favlist_len = favlist_len - 2
            romslist_len = romslist_len + 2
            list_len = favlist_len
            offset = offset - 2
          # show next game (or switch to roms-list if favorites is empty)
          if listr == 'favorites' and favlist_len == 0: 
            listr = 'complete'
            list_to_read = roms_list
            list_len = romslist_len
            text = roms_list_text
            wawfocus = 0
          offset = offset + 2
          if offset == list_len: offset = 0
          wawfocus = 0
          if play_sound == 1: button_sound.play()
          show_screen(screen,list_to_read,list_len,offset,text,wawfocus,0)

        # --------------------------------------------------
        # ignore rom
        if wawfocus == 4:
          if listr == 'complete':
            temp_int = offset
            game = list_to_read[temp_int]
            roms_list.pop(temp_int)
            title = list_to_read[temp_int]
            roms_list.pop(temp_int)
            cmd = 'mv '+complete+'/'+game+'.zip '+ignore
            os.popen(cmd)
            romslist_len = romslist_len - 2
            list_len = romslist_len
            offset = offset - 2
            # show next game
            offset = offset + 2
            if offset == list_len: offset = 0
            wawfocus = 0
            if play_sound == 1: button_sound.play()
            show_screen(screen,list_to_read,list_len,offset,text,wawfocus,0)

        # --------------------------------------------------
        # shutdown machine
        if wawfocus == 5:
          cmd = 'sudo /sbin/shutdown -h now'
          os.system(cmd)

      # --------------------------------------------------
      # key up // down : change game
      if event.key == K_UP or event.key == K_DOWN:
	if wawfocus == 0:
          # up or down ?
	  if event.key == K_UP:
	    offset = offset - 2
            if offset == -2: offset = list_len - 2
          if event.key == K_DOWN:
            offset = offset + 2
	    if offset == list_len: offset = 0
          if play_sound == 1: button_sound.play()
          show_screen(screen,list_to_read,list_len,offset,text,wawfocus,0)

      # --------------------------------------------------
      # key right // down: change focus
      if event.key == K_LEFT or event.key == K_RIGHT:
        if event.key == K_LEFT:
          wawfocus = wawfocus - 1
          if wawfocus == -1: wawfocus = waw_len - 1
          if listr == 'favorites' and wawfocus == 4: wawfocus = wawfocus - 1
        if event.key == K_RIGHT:
          wawfocus = wawfocus + 1
          if listr == 'favorites' and wawfocus == 4: wawfocus = wawfocus + 1
          if wawfocus == waw_len: wawfocus = 0
        if play_sound == 1: button_sound.play()
        focustext = waw_text[wawfocus]        
        if wawfocus == 2:
          tocheck = romsdir+"/scaling/"+list_to_read[offset]+".zip_2"
          check = os.path.isfile(tocheck)
          if check == False:
            focustext += ": 1"
            scaling = 1
          else:
            focustext += ": 2"
            scaling = 2
        show_screen(screen,list_to_read,list_len,offset,text,wawfocus,focustext)

    # --------------------------------------------------
    # first run!
    if first_run == 1:
      show_screen(screen,list_to_read,list_len,offset,text,wawfocus,0)
      first_run = 0

  pygame.quit()

if __name__=='__main__': main()
